<?php
/**
 * Room Slider Template
 * Hoteller Elementor Extension
 *
 * Este archivo funciona como controlador para determinar qué estilo de slider cargar
 * basándose en la configuración seleccionada en el widget Elementor.
 */

// Salir si se accede directamente
if (!defined('ABSPATH')) {
    exit;
}

// Aumentar el límite de memoria temporalmente
ini_set('memory_limit', '256M');

// Obtener todas las configuraciones
$settings = $this->get_settings_for_display();

// Verificar si MotoPress Hotel Booking está activo
if (!class_exists('MPHB') || !post_type_exists('mphb_room_type')) {
    echo '<div class="room-slider-error">' .
        esc_html__('MotoPress Hotel Booking plugin is required for this widget to work.', 'hoteller-elementor') .
        '</div>';
    return;
}

// Consultar habitaciones - Limitar a un máximo de 6 habitaciones para evitar problemas de memoria
$posts_per_page = isset($settings['posts_per_page']) ? intval($settings['posts_per_page']) : 6;
// Nunca permitir -1 (todas las habitaciones) para prevenir agotamiento de memoria
if ($posts_per_page < 1 || $posts_per_page > 12) {
    $posts_per_page = 6;
}

$args = array(
    'post_type' => 'mphb_room_type',
    'posts_per_page' => $posts_per_page,
    'post_status' => 'publish',
    'orderby' => $settings['orderby'] ?? 'date',
    'order' => $settings['order'] ?? 'DESC',
    // Optimizaciones para reducir consultas adicionales
    'update_post_meta_cache' => true,
    'update_post_term_cache' => true,
);

// Si se seleccionaron habitaciones específicas
if (isset($settings['room_selection_type']) && $settings['room_selection_type'] === 'specific' && !empty($settings['specific_rooms'])) {
    // Asegurar que specific_rooms está correctamente procesado
    $selected_rooms = $settings['specific_rooms'];
    
    // Si es una cadena o un array con comas, convertir a array
    if (is_string($selected_rooms) && strpos($selected_rooms, ',') !== false) {
        $selected_rooms = explode(',', $selected_rooms);
    }
    
    // Si no es un array, convertirlo
    if (!is_array($selected_rooms)) {
        $selected_rooms = array($selected_rooms);
    }
    
    // Limpiar y validar IDs
    $clean_ids = array_map('intval', $selected_rooms);
    $clean_ids = array_filter($clean_ids); // Eliminar valores 0
    
    if (!empty($clean_ids)) {
        // Limitar a un máximo de 6 habitaciones para prevenir problemas de memoria
        $clean_ids = array_slice($clean_ids, 0, 6);
        $args['post__in'] = $clean_ids;
        $args['orderby'] = 'post__in'; // Preservar orden de selección
    }
}

// Si hay categorías seleccionadas, añadirlas a la consulta
if (isset($settings['room_selection_type']) && $settings['room_selection_type'] === 'all' && 
    !empty($settings['room_categories']) && is_array($settings['room_categories'])) {
    $args['tax_query'] = array(
        array(
            'taxonomy' => 'mphb_room_type_category',
            'field' => 'term_id',
            'terms' => $settings['room_categories'],
        )
    );
}

// Realizar la consulta
$rooms = new WP_Query($args);

// Si no hay resultados, mostrar mensaje
if (!$rooms->have_posts()) {
    echo '<div class="room-slider-no-rooms">' . 
        esc_html__('No rooms found. Please add rooms or adjust your selection criteria.', 'hoteller-elementor');
        
    // Agregar información de depuración para administradores
    if (current_user_can('manage_options')) {
        echo '<div class="room-slider-debug-info">';
        echo '<p><strong>' . esc_html__('Debug Information:', 'hoteller-elementor') . '</strong></p>';
        echo '<p>' . esc_html__('Query Arguments:', 'hoteller-elementor') . '</p>';
        echo '<pre>' . esc_html(print_r($args, true)) . '</pre>';
        echo '</div>';
    }
    
    echo '</div>';
    return;
}

// Verificar el formato de slider_style (podría ser "style1" o "1")
$style = isset($settings['slider_style']) ? $settings['slider_style'] : 'style1';

// Si es un número, añadir el prefijo "style"
if (is_numeric($style)) {
    $style = 'style' . $style;
}

// Asegurarse de que el estilo sea válido
if (!in_array($style, ['style1', 'style2', 'style3'])) {
    $style = 'style1';
}

// Mapear el estilo a la clase CSS correspondiente
$style_class_map = [
    'style1' => 'modern-style', 
    'style2' => 'style2',
    'style3' => 'style3'
];

// Usar la clase CSS correspondiente
$style_class = $style_class_map[$style];

// Verificar si el archivo de estilo existe
$style_file = __DIR__ . '/' . $style . '.php';

// Verificar que el archivo existe antes de incluirlo
if (file_exists($style_file)) {
    // Precargar datos comunes para reducir consultas dentro del bucle
    if ($rooms->post_count > 0) {
        // Obtener IDs de todas las habitaciones para precarga
        $room_ids = wp_list_pluck($rooms->posts, 'ID');
        
        // Precargar amenidades para todas las habitaciones
        if (!empty($room_ids) && $settings['show_amenities'] === 'yes') {
            $amenities_by_room = [];
            foreach ($room_ids as $id) {
                $terms = get_the_terms($id, 'mphb_room_type_facility');
                if (!empty($terms) && !is_wp_error($terms)) {
                    $amenities_by_room[$id] = $terms;
                }
            }
        }
        
        // Precargar precios personalizados
        if (!empty($room_ids) && $settings['show_price'] === 'yes') {
            $custom_prices = [];
            foreach ($room_ids as $id) {
                $custom_prices[$id] = get_post_meta($id, 'custom_pricing', true);
            }
        }
    }
    
    // Incluir el archivo de estilo correspondiente
    include $style_file;
} else {
    echo '<div class="room-slider-error">' . 
        esc_html__('Style file not found:', 'hoteller-elementor') . ' ' . esc_html($style) . 
        '</div>';
}

// Liberar memoria
if (isset($amenities_by_room)) unset($amenities_by_room);
if (isset($custom_prices)) unset($custom_prices);
if (isset($room_ids)) unset($room_ids);

// Restablecer datos de post después de la consulta
wp_reset_postdata();