<?php
namespace HotellerElementor\Widgets;

use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Group_Control_Typography;
use Elementor\Group_Control_Border;
use Elementor\Core\Kits\Documents\Tabs\Global_Colors;
use Elementor\Core\Kits\Documents\Tabs\Global_Typography;

if (!defined('ABSPATH')) exit;

/**
 * Hoteller Room Slider Widget
 *
 * Displays hotel rooms in an interactive slider with various styles and options.
 *
 * @since 1.0.0
 */
class Hoteller_Room_Slider extends Widget_Base {
    /**
     * Get widget name.
     *
     * @since 1.0.0
     * @access public
     * @return string Widget name.
     */
    public function get_name() {
        return 'hoteller-room-slider';
    }

    /**
     * Get widget title.
     *
     * @since 1.0.0
     * @access public
     * @return string Widget title.
     */
    public function get_title() {
        return esc_html__('Room Slider', 'hoteller-elementor');
    }

    /**
     * Get widget icon.
     *
     * @since 1.0.0
     * @access public
     * @return string Widget icon.
     */
    public function get_icon() {
        return 'eicon-slides';
    }

    /**
     * Get widget categories.
     *
     * @since 1.0.0
     * @access public
     * @return array Widget categories.
     */
    public function get_categories() {
        return ['hoteller-theme-widgets-category'];
    }

    /**
     * Get widget script dependencies.
     *
     * @since 1.0.0
     * @access public
     * @return array Widget script dependencies.
     */
    public function get_script_depends() {
        return ['swiper', 'hoteller-elementor'];
    }

    /**
     * Get widget style dependencies.
     *
     * @since 1.0.0
     * @access public
     * @return array Widget style dependencies.
     */
    public function get_style_depends() {
        return ['room-slider'];
    }

    /**
     * Get available room types.
     *
     * @since 1.0.0
     * @access private
     * @return array Available room types.
     */
    private function get_available_rooms() {
        $room_options = [];
        
        // Intentar consultar las habitaciones sin verificacin previa
        try {
            $rooms_query = new \WP_Query([
                'post_type' => 'mphb_room_type',
                'posts_per_page' => -1,
                'orderby' => 'title',
                'order' => 'ASC',
                'post_status' => 'publish'
            ]);
            
            if ($rooms_query->have_posts()) {
                while ($rooms_query->have_posts()) {
                    $rooms_query->the_post();
                    $room_options[get_the_ID()] = get_the_title();
                }
            }
            wp_reset_postdata();
        } catch (\Exception $e) {
            // Manejar error silenciosamente
        }
        
        return $room_options;
    }

    /**
     * Get available room categories.
     *
     * @since 1.0.0
     * @access private
     * @return array Available room categories.
     */
    private function get_room_categories() {
        $categories = [];
        
        if (!taxonomy_exists('mphb_room_type_category')) {
            return $categories;
        }
        
        $terms = get_terms([
            'taxonomy' => 'mphb_room_type_category',
            'hide_empty' => false,
        ]);
        
        if (is_wp_error($terms) || empty($terms)) {
            return $categories;
        }
        
        foreach ($terms as $term) {
            $categories[$term->term_id] = $term->name;
        }
        
        return $categories;
    }

    /**
     * Register widget controls.
     *
     * @since 1.0.0
     * @access protected
     */
    protected function register_controls() {
        // Content Section
        $this->start_controls_section(
            'section_content',
            [
                'label' => esc_html__('Content', 'hoteller-elementor'),
            ]
        );

        $this->add_control(
            'slider_style',
            [
                'label' => esc_html__('Style', 'hoteller-elementor'),
                'type' => Controls_Manager::SELECT,
                'default' => 'style1',
                'options' => [
                    'style1' => esc_html__('Modern', 'hoteller-elementor'),
                    'style2' => esc_html__('Classic', 'hoteller-elementor'),
                    'style3' => esc_html__('Minimal', 'hoteller-elementor'),
                ],
            ]
        );

        $this->add_control(
            'room_selection_type',
            [
                'label' => esc_html__('Room Selection', 'hoteller-elementor'),
                'type' => Controls_Manager::SELECT,
                'default' => 'all',
                'options' => [
                    'all' => esc_html__('All Rooms', 'hoteller-elementor'),
                    'specific' => esc_html__('Specific Rooms', 'hoteller-elementor'),
                ],
            ]
        );

        $room_options = $this->get_available_rooms();
        
        $this->add_control(
            'specific_rooms',
            [
                'label' => esc_html__('Select Rooms', 'hoteller-elementor'),
                'type' => Controls_Manager::SELECT2,
                'multiple' => true,
                'options' => $room_options,
                'condition' => [
                    'room_selection_type' => 'specific'
                ]
            ]
        );

        $this->add_control(
            'room_categories',
            [
                'label' => esc_html__('Room Categories', 'hoteller-elementor'),
                'type' => Controls_Manager::SELECT2,
                'multiple' => true,
                'options' => $this->get_room_categories(),
                'condition' => [
                    'room_selection_type' => 'all'
                ]
            ]
        );

        $this->add_control(
            'posts_per_page',
            [
                'label' => esc_html__('Number of Rooms', 'hoteller-elementor'),
                'type' => Controls_Manager::NUMBER,
                'default' => 6,
                'min' => 1,
                'max' => 20,
                'condition' => [
                    'room_selection_type' => 'all'
                ]
            ]
        );

        $this->add_control(
            'orderby',
            [
                'label' => esc_html__('Order By', 'hoteller-elementor'),
                'type' => Controls_Manager::SELECT,
                'default' => 'date',
                'options' => [
                    'date' => esc_html__('Date', 'hoteller-elementor'),
                    'title' => esc_html__('Title', 'hoteller-elementor'),
                    'menu_order' => esc_html__('Menu Order', 'hoteller-elementor'),
                    'rand' => esc_html__('Random', 'hoteller-elementor'),
                ],
            ]
        );

        $this->add_control(
            'order',
            [
                'label' => esc_html__('Order', 'hoteller-elementor'),
                'type' => Controls_Manager::SELECT,
                'default' => 'DESC',
                'options' => [
                    'ASC' => esc_html__('Ascending', 'hoteller-elementor'),
                    'DESC' => esc_html__('Descending', 'hoteller-elementor'),
                ],
            ]
        );

        $this->add_control(
            'content_options_heading',
            [
                'label' => esc_html__('Content Options', 'hoteller-elementor'),
                'type' => Controls_Manager::HEADING,
                'separator' => 'before',
            ]
        );

        $this->add_control(
            'show_description',
            [
                'label' => esc_html__('Show Description', 'hoteller-elementor'),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'yes',
                'label_on' => esc_html__('Yes', 'hoteller-elementor'),
                'label_off' => esc_html__('No', 'hoteller-elementor'),
                'return_value' => 'yes',
            ]
        );

        $this->add_control(
            'description_length',
            [
                'label' => esc_html__('Description Length', 'hoteller-elementor'),
                'type' => Controls_Manager::NUMBER,
                'default' => 120,
                'min' => 10,
                'max' => 500,
                'condition' => [
                    'show_description' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'show_amenities',
            [
                'label' => esc_html__('Show Amenities', 'hoteller-elementor'),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'yes',
                'label_on' => esc_html__('Yes', 'hoteller-elementor'),
                'label_off' => esc_html__('No', 'hoteller-elementor'),
                'return_value' => 'yes',
            ]
        );

        $this->add_control(
            'show_price',
            [
                'label' => esc_html__('Show Price', 'hoteller-elementor'),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'yes',
                'label_on' => esc_html__('Yes', 'hoteller-elementor'),
                'label_off' => esc_html__('No', 'hoteller-elementor'),
                'return_value' => 'yes',
            ]
        );

        $this->add_control(
            'show_custom_text',
            [
                'label' => esc_html__('Show Custom Text Below Price', 'hoteller-elementor'),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'yes',
                'label_on' => esc_html__('Yes', 'hoteller-elementor'),
                'label_off' => esc_html__('No', 'hoteller-elementor'),
                'return_value' => 'yes',
                'condition' => [
                    'show_price' => 'yes',
                ],
            ]
        );

        $this->add_control(
            'custom_text_below_price',
            [
                'label' => esc_html__('Text Below Price', 'hoteller-elementor'),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__('per night', 'hoteller-elementor'),
                'placeholder' => esc_html__('per night', 'hoteller-elementor'),
                'label_block' => true,
                'condition' => [
                    'show_custom_text' => 'yes',
                    'show_price' => 'yes',
                ],
            ]
        );

        $this->add_control(
            'slider_options_heading',
            [
                'label' => esc_html__('Slider Options', 'hoteller-elementor'),
                'type' => Controls_Manager::HEADING,
                'separator' => 'before',
            ]
        );

        $this->add_control(
            'autoplay',
            [
                'label' => esc_html__('Autoplay', 'hoteller-elementor'),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'yes',
                'label_on' => esc_html__('Yes', 'hoteller-elementor'),
                'label_off' => esc_html__('No', 'hoteller-elementor'),
                'return_value' => 'yes',
            ]
        );

        $this->add_control(
            'autoplay_speed',
            [
                'label' => esc_html__('Autoplay Speed', 'hoteller-elementor'),
                'type' => Controls_Manager::NUMBER,
                'default' => 5000,
                'min' => 1000,
                'max' => 10000,
                'step' => 500,
                'condition' => [
                    'autoplay' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'image_size',
            [
                'label' => esc_html__('Image Size', 'hoteller-elementor'),
                'type' => Controls_Manager::SELECT,
                'default' => 'large',
                'options' => $this->get_image_sizes(),
            ]
        );

        $this->end_controls_section();

        // Description Style Section
        $this->start_controls_section(
            'section_style_description',
            [
                'label' => esc_html__('Description', 'hoteller-elementor'),
                'tab' => Controls_Manager::TAB_STYLE,
                'condition' => [
                    'show_description' => 'yes'
                ]
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'description_typography',
                'label' => esc_html__('Typography', 'hoteller-elementor'),
                'selector' => '{{WRAPPER}} .room_slide_excerpt',
            ]
        );

        $this->add_control(
            'description_color',
            [
                'label' => esc_html__('Text Color', 'hoteller-elementor'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .room_slide_excerpt' => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'description_alignment',
            [
                'label' => esc_html__('Text Alignment', 'hoteller-elementor'),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__('Left', 'hoteller-elementor'),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__('Center', 'hoteller-elementor'),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__('Right', 'hoteller-elementor'),
                        'icon' => 'eicon-text-align-right',
                    ],
                    'justify' => [
                        'title' => esc_html__('Justify', 'hoteller-elementor'),
                       'icon' => 'eicon-text-align-justify',
                   ],
               ],
               'default' => 'left',
               'selectors' => [
                   '{{WRAPPER}} .room_slide_excerpt' => 'text-align: {{VALUE}};',
               ],
           ]
       );

       $this->add_responsive_control(
           'description_margin',
           [
               'label' => esc_html__('Margin', 'hoteller-elementor'),
               'type' => Controls_Manager::DIMENSIONS,
               'size_units' => ['px', 'em', '%'],
               'selectors' => [
                   '{{WRAPPER}} .room_slide_excerpt' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
               ],
           ]
       );

       $this->end_controls_section();

       // Amenities Style Section
       $this->start_controls_section(
           'section_style_amenities',
           [
               'label' => esc_html__('Amenities', 'hoteller-elementor'),
               'tab' => Controls_Manager::TAB_STYLE,
               'condition' => [
                   'show_amenities' => 'yes'
               ]
           ]
       );

       $this->add_control(
           'amenities_color',
           [
               'label' => esc_html__('Text Color', 'hoteller-elementor'),
               'type' => Controls_Manager::COLOR,
               'selectors' => [
                   '{{WRAPPER}} .amenity-item' => 'color: {{VALUE}};',
               ],
           ]
       );

       $this->add_control(
           'amenities_icon_color',
           [
               'label' => esc_html__('Icon Color', 'hoteller-elementor'),
               'type' => Controls_Manager::COLOR,
               'selectors' => [
                   '{{WRAPPER}} .amenity-item i' => 'color: {{VALUE}};',
               ],
           ]
       );

       $this->add_control(
           'amenities_icon_size',
           [
               'label' => esc_html__('Icon Size', 'hoteller-elementor'),
               'type' => Controls_Manager::SLIDER,
               'size_units' => ['px', 'em', 'rem'],
               'range' => [
                   'px' => [
                       'min' => 12,
                       'max' => 48,
                       'step' => 1,
                   ],
               ],
               'default' => [
                   'unit' => 'px',
                   'size' => 16,
               ],
               'selectors' => [
                   '{{WRAPPER}} .amenity-item i' => 'font-size: {{SIZE}}{{UNIT}};',
               ],
           ]
       );

       $this->add_responsive_control(
           'amenities_spacing',
           [
               'label' => esc_html__('Item Spacing', 'hoteller-elementor'),
               'type' => Controls_Manager::SLIDER,
               'size_units' => ['px', 'em'],
               'range' => [
                   'px' => [
                       'min' => 0,
                       'max' => 50,
                       'step' => 1,
                   ],
               ],
               'selectors' => [
                   '{{WRAPPER}} .amenity-item' => 'margin-right: {{SIZE}}{{UNIT}}; margin-bottom: {{SIZE}}{{UNIT}};',
               ],
           ]
       );

       $this->end_controls_section();

       // Price Style Section
       $this->start_controls_section(
           'section_style_price',
           [
               'label' => esc_html__('Price', 'hoteller-elementor'),
               'tab' => Controls_Manager::TAB_STYLE,
               'condition' => [
                   'show_price' => 'yes'
               ]
           ]
       );

       $this->add_control(
           'price_color',
           [
               'label' => esc_html__('Price Color', 'hoteller-elementor'),
               'type' => Controls_Manager::COLOR,
               'selectors' => [
                   '{{WRAPPER}} .mphb-price' => 'color: {{VALUE}};',
               ],
           ]
       );

       $this->add_responsive_control(
           'price_size',
           [
               'label' => esc_html__('Price Size', 'hoteller-elementor'),
               'type' => Controls_Manager::SLIDER,
               'size_units' => ['px', 'em', 'rem'],
               'range' => [
                   'px' => [
                       'min' => 12,
                       'max' => 100,
                       'step' => 1,
                   ],
               ],
               'default' => [
                   'unit' => 'px',
                   'size' => 28,
               ],
               'selectors' => [
                   '{{WRAPPER}} .mphb-price' => 'font-size: {{SIZE}}{{UNIT}};',
               ],
           ]
       );

       $this->add_control(
           'price_period_color',
           [
               'label' => esc_html__('Period Text Color', 'hoteller-elementor'),
               'type' => Controls_Manager::COLOR,
               'selectors' => [
                   '{{WRAPPER}} .mphb-price-period' => 'color: {{VALUE}};',
               ],
           ]
       );

       $this->add_group_control(
           Group_Control_Typography::get_type(),
           [
               'name' => 'custom_text_typography',
               'label' => esc_html__('Custom Text Typography', 'hoteller-elementor'),
               'selector' => '{{WRAPPER}} .custom-text-below-price',
               'condition' => [
                   'show_custom_text' => 'yes',
               ],
           ]
       );

       $this->add_control(
           'custom_text_color',
           [
               'label' => esc_html__('Custom Text Color', 'hoteller-elementor'),
               'type' => Controls_Manager::COLOR,
               'selectors' => [
                   '{{WRAPPER}} .custom-text-below-price' => 'color: {{VALUE}};',
               ],
               'condition' => [
                   'show_custom_text' => 'yes',
               ],
           ]
       );

       $this->end_controls_section();

       // Button Style Section
       $this->start_controls_section(
           'section_style_button',
           [
               'label' => esc_html__('Button', 'hoteller-elementor'),
               'tab' => Controls_Manager::TAB_STYLE,
           ]
       );

       $this->add_control(
           'button_text_color',
           [
               'label' => esc_html__('Text Color', 'hoteller-elementor'),
               'type' => Controls_Manager::COLOR,
               'selectors' => [
                   '{{WRAPPER}} .button' => 'color: {{VALUE}};',
               ],
           ]
       );

       $this->add_control(
           'button_text_hover_color',
           [
               'label' => esc_html__('Text Hover Color', 'hoteller-elementor'),
               'type' => Controls_Manager::COLOR,
               'selectors' => [
                   '{{WRAPPER}} .button:hover' => 'color: {{VALUE}};',
               ],
           ]
       );

       $this->add_control(
           'button_background_color',
           [
               'label' => esc_html__('Background Color', 'hoteller-elementor'),
               'type' => Controls_Manager::COLOR,
               'selectors' => [
                   '{{WRAPPER}} .button' => 'background-color: {{VALUE}};',
               ],
           ]
       );

       $this->add_control(
           'button_background_hover_color',
           [
               'label' => esc_html__('Background Hover Color', 'hoteller-elementor'),
               'type' => Controls_Manager::COLOR,
               'selectors' => [
                   '{{WRAPPER}} .button:hover' => 'background-color: {{VALUE}};',
               ],
           ]
       );

       $this->add_group_control(
           Group_Control_Border::get_type(),
           [
               'name' => 'button_border',
               'label' => esc_html__('Border', 'hoteller-elementor'),
               'selector' => '{{WRAPPER}} .button',
           ]
       );

       $this->add_control(
           'button_border_radius',
           [
               'label' => esc_html__('Border Radius', 'hoteller-elementor'),
               'type' => Controls_Manager::DIMENSIONS,
               'size_units' => ['px', '%', 'em'],
               'selectors' => [
                   '{{WRAPPER}} .button' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
               ],
           ]
       );

       $this->add_control(
           'button_padding',
           [
               'label' => esc_html__('Padding', 'hoteller-elementor'),
               'type' => Controls_Manager::DIMENSIONS,
               'size_units' => ['px', 'em', '%'],
               'selectors' => [
                   '{{WRAPPER}} .button' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
               ],
           ]
       );

       $this->add_group_control(
           Group_Control_Typography::get_type(),
           [
               'name' => 'button_typography',
               'selector' => '{{WRAPPER}} .button',
           ]
       );

       $this->end_controls_section();

       // Navigation Style Section
       $this->start_controls_section(
           'section_style_navigation',
           [
               'label' => esc_html__('Navigation', 'hoteller-elementor'),
               'tab' => Controls_Manager::TAB_STYLE,
           ]
       );

       $this->add_control(
           'arrow_color',
           [
               'label' => esc_html__('Arrow Color', 'hoteller-elementor'),
               'type' => Controls_Manager::COLOR,
               'selectors' => [
                   '{{WRAPPER}} .swiper-button-next, {{WRAPPER}} .swiper-button-prev' => 'color: {{VALUE}};',
               ],
           ]
       );

       $this->add_control(
           'arrow_hover_color',
           [
               'label' => esc_html__('Arrow Hover Color', 'hoteller-elementor'),
               'type' => Controls_Manager::COLOR,
               'selectors' => [
                   '{{WRAPPER}} .swiper-button-next:hover, {{WRAPPER}} .swiper-button-prev:hover' => 'color: {{VALUE}};',
               ],
           ]
       );

       $this->add_control(
           'arrow_size',
           [
               'label' => esc_html__('Arrow Size', 'hoteller-elementor'),
               'type' => Controls_Manager::SLIDER,
               'size_units' => ['px'],
               'range' => [
                   'px' => [
                       'min' => 20,
                       'max' => 60,
                       'step' => 1,
                   ],
               ],
               'selectors' => [
                   '{{WRAPPER}} .swiper-button-next, {{WRAPPER}} .swiper-button-prev' => 'font-size: {{SIZE}}{{UNIT}};',
               ],
           ]
       );

       $this->add_control(
           'pagination_heading',
           [
               'label' => esc_html__('Pagination', 'hoteller-elementor'),
               'type' => Controls_Manager::HEADING,
               'separator' => 'before',
           ]
       );

       $this->add_control(
           'pagination_color',
           [
               'label' => esc_html__('Pagination Color', 'hoteller-elementor'),
               'type' => Controls_Manager::COLOR,
               'selectors' => [
                   '{{WRAPPER}} .swiper-pagination-bullet' => 'background-color: {{VALUE}};',
               ],
           ]
       );

       $this->add_control(
           'pagination_active_color',
           [
               'label' => esc_html__('Pagination Active Color', 'hoteller-elementor'),
               'type' => Controls_Manager::COLOR,
               'selectors' => [
                   '{{WRAPPER}} .swiper-pagination-bullet-active' => 'background-color: {{VALUE}};',
               ],
           ]
       );

       $this->end_controls_section();
   }

   /**
    * Get available image sizes.
    *
    * @since 1.0.0
    * @access private
    * @return array Available image sizes.
    */
   private function get_image_sizes() {
       $sizes = get_intermediate_image_sizes();
       $options = [
           'full' => esc_html__('Full', 'hoteller-elementor')
       ];
       
       foreach ($sizes as $size) {
           $options[$size] = $size;
       }
       
       return $options;
   }

   /**
    * Render the widget output on the frontend.
    *
    * @since 1.0.0
    * @access protected
    */
   protected function render() {
       $settings = $this->get_settings_for_display();
       
       // Configure the query arguments based on settings
       $args = [
           'post_type' => 'mphb_room_type',
           'posts_per_page' => ($settings['room_selection_type'] === 'all') 
               ? absint($settings['posts_per_page']) 
               : -1,
           'orderby' => $settings['orderby'] ?? 'date',
           'order' => $settings['order'] ?? 'DESC',
           'post_status' => 'publish',
       ];

       // If specific rooms are selected
       if ($settings['room_selection_type'] === 'specific' && !empty($settings['specific_rooms'])) {
           // Ensure specific_rooms is properly processed
           $selected_rooms = $settings['specific_rooms'];
           
           // If it's a string or array with commas, convert to array
           if (is_string($selected_rooms) && strpos($selected_rooms, ',') !== false) {
               $selected_rooms = explode(',', $selected_rooms);
           }
           
           // If not an array, convert it
           if (!is_array($selected_rooms)) {
               $selected_rooms = array($selected_rooms);
           }
           
           // Clean and validate IDs
           $clean_ids = array_map('intval', $selected_rooms);
           $clean_ids = array_filter($clean_ids); // Remove any 0 values
           
           if (!empty($clean_ids)) {
               $args['post__in'] = $clean_ids;
               $args['orderby'] = 'post__in'; // Preserve selection order
           }
       }
       
       // If room categories are selected
       if ($settings['room_selection_type'] === 'all' && !empty($settings['room_categories'])) {
           $args['tax_query'] = array(
               array(
                   'taxonomy' => 'mphb_room_type_category',
                   'field' => 'term_id',
                   'terms' => $settings['room_categories'],
               )
           );
       }
       
       // Execute the query
       $rooms = new \WP_Query($args);
       
       if ($rooms->have_posts()) {
           // Determinar la ruta del archivo de estilo a incluir
           $template_file = HOTELLER_ELEMENTOR_PATH . 'templates/room-slider/' . $settings['slider_style'] . '.php';
           
           // Verificar si el archivo existe
           if (file_exists($template_file)) {
               include($template_file);
           } else {
               // Intentar con el archivo index.php si existe
               $index_file = HOTELLER_ELEMENTOR_PATH . 'templates/room-slider/index.php';
               if (file_exists($index_file)) {
                   include($index_file);
               } else {
                   echo '<div class="room-slider-error">' . 
                       esc_html__('Template file not found.', 'hoteller-elementor') . 
                       '</div>';
               }
           }
       } else {
           echo '<div class="room-slider-no-rooms">' . 
               esc_html__('No rooms found. Please add rooms or adjust your selection criteria.', 'hoteller-elementor') . 
               '</div>';
       }
       
       // Reset post data to avoid conflicts with other queries
       wp_reset_postdata();
   }
}